/******************************************************************************
 *	File	:	main.c
 *	Version :	1.1
 *	Date	:	2021/01/13
 *	Author	:	XFORCE
 *
 *	Demonstrate how to use ECM-XF control drive type slave device,
 *	And how to configure PDO of slave.
 *
 *
 * @copyright (C) 2021 NEXTW TECHNOLOGY CO., LTD.. All rights reserved.
 *
 ******************************************************************************/

#include "EcmUsrDriver.h"
#include "platform.h"
#include "Utility.h"
#include "PdoDefine.h"

#define TEST_SPI_FREQ			24000000
#define DC_ACTIVE_CODE			0x300
#define BASE_CYCTIME			1000000
/*
 * TEST_CYCTIME_DIVID : 1 / 2 / 4 / 8
 * 1 : 1ms
 * 2 : 0.5ms
 * 4 : 0.25ms
 * 8 : 0.125ms
 * */
#define TEST_CYCTIME_DIVID		4
#define TEST_FEED_SPEED			(200/TEST_CYCTIME_DIVID)
/*
 * Change this define for your need
 * */
#define TEST_SERVO_CNT			4
#define TEST_CONST_TIME			(2000*TEST_CYCTIME_DIVID)
#define TEST_PDO_TO_FIFO_ONCE	TEST_CYCTIME_DIVID

#define TEST_FIFO_CNT			255
//#define TEST_FIFO_CNT			PDO_FIFO_DEFAULT_CNT
//
#define SERVO_ON_STATE			CIA402_SW_OPERATIONENABLED
#define SERVO_OFF_STATE			CIA402_SW_READYTOSWITCHON


uint8_t RxPDOData[TEST_SPI_DATA_SIZE];
uint8_t TxPDOData[TEST_SPI_DATA_SIZE];

int nPos[TEST_SERVO_CNT];
int AlignmentPosition(int nDriveCnt, int *pnPos, int nAxisRxPDOSize, int nAxisTxPDOSize)
{
	int i,j,nWrCnt;
	uint8_t u8Data;
	uint16_t u16PDOSize;
	RXPDO_ST_DEF_T *pRxPDOAxis;
	TXPDO_ST_DEF_T *pTxPDOAxis;
	nWrCnt = 0;
	for(i=0;i<10;i++){
		u16PDOSize = nDriveCnt*nAxisRxPDOSize;
		// Data exchange without FIFO
		u8Data=ECM_EcatPdoDataExchange(ECM_PDO_RW_OP, RxPDOData, TxPDOData, &u16PDOSize);
		if((u8Data & ECM_PDO_RD_OP) == ECM_PDO_RD_OP){
			//Get valid data
			for(j=0;j<nDriveCnt;j++){
				pTxPDOAxis = (TXPDO_ST_DEF_T *)&TxPDOData[j*nAxisTxPDOSize];
				pRxPDOAxis = (RXPDO_ST_DEF_T *)&RxPDOData[j*nAxisRxPDOSize];
				pnPos[j] = pTxPDOAxis->n32AcuPos;
				pRxPDOAxis->n32TarPos = pnPos[j];
			}
		}
		if((u8Data & ECM_PDO_RW_OP) == ECM_PDO_RW_OP){
			//Get valid data and write back
			if(nWrCnt){
				return 1;
			}
			nWrCnt++;
		}
	}
	return 0;
}
int main (int argc,char ** argv)
{
	int i=0,j=0;
	int nVel=0;
	int nret=0;
	int nKey=0;
	uint8_t u8Version=0,u8FifoCnt=0,u8FifoCntMax=TEST_FIFO_CNT;
	uint8_t u8State=0, u8WkcErrCnt=0, u8CrcErrCnt=0;
	uint8_t u8LastState=0, u8FirstWkcErrCnt=0, u8LastCrcErrCnt=0;
	uint16_t u16RxPDOSize=0,u16TxPDOSize=0,u16SpiSize=0;
	int nAxisRxPDOSize=0,nAxisTxPDOSize=0;
	int8_t SlaveCnt = 0;
	RXPDO_ST_DEF_T *pRxPDOAxis;
	TXPDO_ST_DEF_T *pTxPDOAxis;
	//	Hardware platform dependency
	UserSys_Init(TEST_SPI_FREQ);
    printf("Enter \"x\" will skip PDO setup\r\n");
    printf("Enter \"y\" will assign PDO\r\n");
    printf("Others will configure PDO\r\n");
    nKey = getchar();
    u16SpiSize = TEST_SPI_DATA_SIZE;
	u8Version = ECM_InitLibrary(&u16SpiSize);
    if(u8Version == 0){
        printf("ECM_InitLibrary : FAILED\r\n");
    	return -1;
    }
    printf("SPI data size %d, FW Ver(0x%X)\r\n",u16SpiSize,u8Version);
    printf("ServoCnt %d, FIFOCnt %d\r\n",TEST_SERVO_CNT,u8FifoCntMax);
    nret=ECM_EcatInit(DC_ACTIVE_CODE, (BASE_CYCTIME/TEST_CYCTIME_DIVID), 0, 1);
    if(nret<=0){
        printf("ECM_EcatInit : Failed\r\n");
    	return -1;
    }
	SlaveCnt = ECM_EcatSlvCntGet();
	if(SlaveCnt < TEST_SERVO_CNT){
		printf("ERROR : Slave count %d < %d\r\n", SlaveCnt, TEST_SERVO_CNT);
		return -1;
	}
	printf("Found %d slaves\r\n", SlaveCnt);
	nret = ECM_StateCheck(0xFF, EC_STATE_PRE_OP, 2000);
	if(nret == 0){
		return -1;
	}
	printf("=============\r\n");
	printf("PRE-OP state\r\n");
	printf("=============\r\n");
	if(nKey == 'x' || nKey == 'X'){
		printf("Skip PDO setup\r\n");
	}else if(nKey == 'y' || nKey == 'Y'){
		printf("Assign PDO\r\n");
		ConfigDrive(1, TEST_SERVO_CNT, 1);
	}else{
		printf("Configure PDO\r\n");
		ConfigDrive(1, TEST_SERVO_CNT, 0);
	}
	if(nKey == '1'){
		u8FifoCntMax = 16;
	}
	ECM_SetTxFIFOCnt(u8FifoCntMax);
	ECM_SetRxFIFOCnt(u8FifoCntMax);
    nret=ECM_CheckMEMSpace(TEST_PDO_TO_FIFO_ONCE);
    if(nret<=0){
        printf("ECM_CheckMEMSpace : Failed\r\n");
    	return -1;
    }
    printf("ECM_CheckMEMSpace : Pass\r\n");
    // Check ECM queue size first
	ECM_InitFIFO();

	u16RxPDOSize = ECM_FifoRxPdoSizeGet();
	u16TxPDOSize = ECM_FifoTxPdoSizeGet();
	nAxisRxPDOSize = u16RxPDOSize/SlaveCnt;
	nAxisTxPDOSize = u16TxPDOSize/SlaveCnt;
	printf("RxPDO size %d, Axis RxPDO size %d\r\n", u16RxPDOSize, nAxisRxPDOSize);
	printf("TxPDO size %d, Axis TxPDO size %d\r\n", u16TxPDOSize, nAxisTxPDOSize);

	nret = SetDriverCmdOpMode(TEST_SERVO_CNT, 8);
	if(nret <= 0){
		return -1;
	}
	nret = ECM_StateCheck(0xFF, EC_STATE_SAFE_OP, 2000);
	if(nret == 0){
		return -1;
	}
	printf("=============\r\n");
	printf("SAFE-OP state\r\n");
	printf("=============\r\n");
	nret = AlignmentPosition(TEST_SERVO_CNT, nPos, nAxisRxPDOSize, nAxisTxPDOSize);
	if(nret <= 0){
		return -1;
	}
	printf("Position alignment : Done\r\n");
	for(i=0; i < TEST_SERVO_CNT ; i++){
		nret = ECM_Drv402SM_AdvConfig(i, i, 0, 0);
		if(nret == 0){
			printf("ECM_Drv402SM_AdvConfig error : %d\r\n",i);
			return -1;
		}
		nret = ECM_Drv402SM_StateSet(i, SERVO_OFF_STATE);
		if(nret == 0){
			printf("ECM_Drv402SM_StateSet error : %d\r\n",i);
			return -1;
		}
	}
	printf("Enable control CiA402 state : Done\r\n");
	nret = ECM_StateCheck(0xFF, EC_STATE_OPERATIONAL, 2000);
	if(nret == 0){
		return -1;
	}
	printf("=============\r\n");
	printf("OP state\r\n");
	printf("=============\r\n");
	printf("Enter any key for SERVO-ON\r\n");
	getchar();
	for(i=0; i < TEST_SERVO_CNT ; i++){
		nret = ECM_Drv402SM_StateSet(i, SERVO_ON_STATE);
		if(nret == 0){
			printf("ECM_Drv402SM_StateSet error : %d\r\n",i);
			return -1;
		}
	}
	printf("Enter any key for start motion example\r\n");
	getchar();
	//
	//	PDO example : use FIFO do cyclic data exchange
	//
	printf("Start user cyclic application\r\n");
	while(1){
		if(u8FifoCnt < (u8FifoCntMax-(2*TEST_PDO_TO_FIFO_ONCE))){
			for(i=0;i<=TEST_PDO_TO_FIFO_ONCE;i++){
				nVel = CalVel(TEST_CYCTIME_DIVID, TEST_FEED_SPEED, TEST_CONST_TIME);
				for(j=0; j<TEST_SERVO_CNT ; j++){
					nPos[j] += nVel;
					pRxPDOAxis = (RXPDO_ST_DEF_T *)&RxPDOData[(i*u16RxPDOSize)+(j*nAxisRxPDOSize)];
					pRxPDOAxis->n32TarPos = nPos[j];
					pRxPDOAxis->n8OpMode = 8;
				}
			}
			nret = ECM_EcatPdoFifoDataExchangeAdv(TEST_PDO_TO_FIFO_ONCE, u8FifoCntMax, RxPDOData, TxPDOData, u16RxPDOSize, &u8FifoCnt, &u8CrcErrCnt, &u8WkcErrCnt);
		}else{
			nret = ECM_InfoUpdate(&u8State, &u8FifoCnt, &u8CrcErrCnt, &u8WkcErrCnt);
			if(nret>0 && u8FifoCnt > 50){
				printf("FIFO(%03d)\t", u8FifoCnt);
				for(i=0; i < TEST_SERVO_CNT ; i++){
					pTxPDOAxis = (TXPDO_ST_DEF_T *)&TxPDOData[i*nAxisTxPDOSize];
					printf("%09d\t", pTxPDOAxis->n32AcuPos);
				}
				printf("\r");
			}
		}
		if(nret > 0){
			if(u8FirstWkcErrCnt == 0){
				u8FirstWkcErrCnt = u8WkcErrCnt;
			}
			// Check EtherCAT communication quality
			if(u8WkcErrCnt >= u8FirstWkcErrCnt){
				if((u8WkcErrCnt - u8FirstWkcErrCnt) > 10){
					printf("\r\nWKC continue error EXIT %d\r\n",u8WkcErrCnt);
					break;
				}
			}else{
				if((0xFF-u8FirstWkcErrCnt+u8WkcErrCnt) > 10){
					printf("\r\nWKC continue error EXIT %d\r\n",u8WkcErrCnt);
					break;
				}
			}
			// Check SPI communication quality
			if(u8LastCrcErrCnt != u8CrcErrCnt){
				printf("\r\nu8CrcErrCnt %d\r\n",u8CrcErrCnt);
			}
			u8LastCrcErrCnt = u8CrcErrCnt;
		}
	}
	UserSys_Close();
}
//	(C) COPYRIGHT 2020 NEXTW Technology Corp.
